// #include <iostream>

#include <amdis/common/ClonablePtr.hpp>

#include "Tests.hpp"

using namespace AMDiS;

int main()
{
  using C = ClonablePtr<int>;

  // constructors
  C p1; // default constructor
  AMDIS_TEST( !(p1) );
  AMDIS_TEST( (p1.get() == nullptr) ); // same as above

  int i = 10;
  C p2(i); // constructor from reference
  AMDIS_TEST( (p2.get() != nullptr) );
  AMDIS_TEST_EQ( *p2 , i );

  p1 = p2; // copy and move assignment
  AMDIS_TEST( (p1.get() != nullptr) );
  AMDIS_TEST_EQ( *p1 , i );
  AMDIS_TEST_EQ( *p1 , *p2 );

  int j = 20;
  auto u = std::make_unique<int>(j);
  AMDIS_TEST( (u.get() != nullptr) );

  C p3(u); // constructor from std::unique_ptr
  AMDIS_TEST( (u.get() == nullptr) ); // unique_ptr is reset to NULL
  AMDIS_TEST( (p3.get() != nullptr) );
  AMDIS_TEST_EQ( *p3 , j );

  C p4(p3); // copy constructor
  AMDIS_TEST( (p3.get() != nullptr) ); // ClonablePtr is not reset
  AMDIS_TEST( (p4.get() != nullptr) );
  AMDIS_TEST_EQ( *p4 , j );
  AMDIS_TEST_EQ( *p3 , *p4 );

  int k = 30;
  C p5 = C::make(k); // factory method
  AMDIS_TEST( (p5.get() != nullptr) );
  AMDIS_TEST_EQ( *p5 , k );

  C p6(*p5); // move constructor
  AMDIS_TEST( (p6.get() != nullptr) );
  AMDIS_TEST_EQ( *p6 , k );
  AMDIS_TEST_EQ( *p5 , *p6 );

/* // Test output
  std::cout << "*p1:" << *p1 << ", " << p1.get() << "\n"
            << "*p2:" << *p2 << ", " << p2.get() << "\n"
            << "*p3:" << *p3 << ", " << p3.get() << "\n"
            << "*p4:" << *p4 << ", " << p4.get() << "\n"
            << "*p5:" << *p5 << ", " << p5.get() << "\n"
            << "*p6:" << *p6 << ", " << p6.get() << "\n"
            << " u :    " << u.get() << "\n"
            << " i :" << i << "\n" 
            << " j :" << j << "\n" 
            << " k :" << k << "\n" 
            << "End of Test" << "\n";
*/
}